@@ -1,4 +1,4 @@
 #! /usr/bin/env bash
 # SPDX-License-Identifier: MIT
 
-$XGETTEXT *.cc -o $podir/kio5_info.pot
+$XGETTEXT *.cpp -o $podir/kio5_info.pot
