@@ -10,7 +10,7 @@
 
 #include "mediacontrols.h"
 #define TRANSLATION_DOMAIN "kfileaudiopreview5"
-#include <klocalizedstring.h>
+#include <KLocalizedString>
 #include <phonon/volumeslider.h>
 #include <phonon/seekslider.h>
 #include <QToolButton>
