@@ -20,8 +20,10 @@
 #include <sys/types.h>
 #include <utime.h>
 
+#include "config-mtp.h"
 #include "kmtpdeviceinterface.h"
 #include "kmtpstorageinterface.h"
+#include "listerinterface.h"
 
 // Pseudo plugin class to embed meta data
 class KIOPluginForMetaData : public QObject
@@ -233,8 +235,34 @@ WorkerResult MTPWorker::listDir(const QUrl &url)
         return WorkerResult::fail(ERR_CANNOT_ENTER_DIRECTORY, url.path());
     }
 
-    int result = 0;
     const QString path = convertPath(url.path());
+
+#ifdef HAVE_LIBMTP_Get_Children
+    const auto pathVariant = storage->getFilesAndFolders2(path);
+    if (std::holds_alternative<QDBusError>(pathVariant)) {
+        qCWarning(LOG_KIO_MTP) << "[ERROR] :: Failed to get lister dbus path" << std::get<QDBusError>(pathVariant);
+        return WorkerResult::fail();
+    }
+    Q_ASSERT(std::holds_alternative<QDBusObjectPath>(pathVariant));
+
+    OrgKdeKmtpListerInterface lister(QStringLiteral("org.kde.kmtpd5"), std::get<QDBusObjectPath>(pathVariant).path(), QDBusConnection::sessionBus(), this);
+
+    QEventLoop loop;
+    connect(&lister, &OrgKdeKmtpListerInterface::entry, this, [this, &lister](const KMTPFile &file) {
+        listEntries({getEntry(file)});
+        if (wasKilled()) {
+            lister.abort(); // issues finished which causes the loop to quit
+        }
+    });
+    connect(&lister, &OrgKdeKmtpListerInterface::finished, &loop, &QEventLoop::quit);
+
+    lister.run();
+    loop.exec();
+
+    qCDebug(LOG_KIO_MTP) << "[SUCCESS]";
+    return WorkerResult::pass();
+#else
+    int result = 0;
     const KMTPFileList files = storage->getFilesAndFolders(path, result);
 
     switch (result) {
@@ -248,9 +276,9 @@ WorkerResult MTPWorker::listDir(const QUrl &url)
     case 2:
         return WorkerResult::fail(ERR_IS_FILE, url.path());
     }
-
     // path not found
     return WorkerResult::fail(ERR_CANNOT_ENTER_DIRECTORY, url.path());
+#endif
 }
 
 WorkerResult MTPWorker::stat(const QUrl &url)
@@ -695,22 +723,15 @@ WorkerResult MTPWorker::del(const QUrl &url, bool)
 
 WorkerResult MTPWorker::rename(const QUrl &src, const QUrl &dest, JobFlags flags)
 {
-    switch (checkUrl(src)) {
-    case Url::Valid:
-        break;
-    case Url::Redirected:
-    case Url::NotFound:
-    case Url::Invalid:
-        return WorkerResult::fail(ERR_MALFORMED_URL, src.path());
-    }
-
-    switch (checkUrl(dest)) {
-    case Url::Valid:
-        break;
-    case Url::Redirected:
-    case Url::NotFound:
-    case Url::Invalid:
-        return WorkerResult::fail(ERR_MALFORMED_URL, dest.path());
+    for (const auto &url : {src, dest}) {
+        switch (checkUrl(src)) {
+        case Url::Valid:
+            break;
+        case Url::Redirected:
+        case Url::NotFound:
+        case Url::Invalid:
+            return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
+        }
     }
 
     if (src.scheme() != QLatin1String("mtp")) {
@@ -722,45 +743,55 @@ WorkerResult MTPWorker::rename(const QUrl &src, const QUrl &dest, JobFlags flags
 
     const QStringList srcItems = src.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
     KMTPDeviceInterface *mtpDevice = m_kmtpDaemon.deviceFromName(srcItems.first());
-    if (mtpDevice) {
-        // rename Device
-        if (srcItems.size() == 1) {
-            const int result = mtpDevice->setFriendlyName(urlFileName(dest));
-            if (!result) {
-                return WorkerResult::pass();
-            }
-        }
-        // rename Storage
-        else if (srcItems.size() == 2) {
+    if (!mtpDevice) {
+        return WorkerResult::fail(ERR_CANNOT_RENAME, src.path());
+    }
+
+    // rename Device
+    if (srcItems.size() == 1) {
+        if (mtpDevice->setFriendlyName(urlFileName(dest)) != 0) {
             return WorkerResult::fail(ERR_CANNOT_RENAME, src.path());
-        } else {
-            // rename file or folder
-            const KMTPStorageInterface *storage = mtpDevice->storageFromDescription(srcItems.at(1));
-            if (storage) {
+        }
+        return WorkerResult::pass();
+    }
 
-                // check if the file already exists on the device
-                const QString destinationPath = convertPath(dest.path());
-                const KMTPFile destinationFile = storage->getFileMetadata(destinationPath);
-                if (destinationFile.isValid()) {
-                    if (flags & KIO::Overwrite) {
-                        // delete existing file on the device
-                        const int result = storage->deleteObject(destinationPath);
-                        if (result) {
-                            return WorkerResult::fail(ERR_CANNOT_DELETE, dest.path());
-                        }
-                    } else {
-                        return WorkerResult::fail(ERR_FILE_ALREADY_EXIST, dest.path());
-                    }
-                }
+    // rename Storage
+    if (srcItems.size() == 2) {
+        return WorkerResult::fail(ERR_CANNOT_RENAME, src.path());
+    }
 
-                const int result = storage->setFileName(convertPath(src.path()), dest.fileName());
-                if (!result) {
-                    return WorkerResult::pass();
-                }
+    // rename across folder boundary -> let KIO fall back to get-put instead.
+    // There is LIBMTP_Move_Object but it's supposedly not supported on all devices. Plus get-put requires no extra code.
+    const auto srcDir = QFileInfo(src.path()).dir().path();
+    const auto dstDir = QFileInfo(dest.path()).dir().path();
+    if (srcDir != dstDir) {
+        return WorkerResult::fail(ERR_UNSUPPORTED_ACTION, src.path());
+    }
+
+    // rename file or folder
+    const KMTPStorageInterface *storage = mtpDevice->storageFromDescription(srcItems.at(1));
+    if (!storage) {
+        return WorkerResult::fail(ERR_CANNOT_RENAME, src.path());
+    }
+
+    // check if the file already exists on the device
+    const QString destinationPath = convertPath(dest.path());
+    const KMTPFile destinationFile = storage->getFileMetadata(destinationPath);
+    if (destinationFile.isValid()) {
+        if (flags & KIO::Overwrite) {
+            // delete existing file on the device
+            if (storage->deleteObject(destinationPath) != 0) {
+                return WorkerResult::fail(ERR_CANNOT_DELETE, dest.path());
             }
+        } else {
+            return WorkerResult::fail(ERR_FILE_ALREADY_EXIST, dest.path());
         }
     }
-    return WorkerResult::fail(ERR_CANNOT_RENAME, src.path());
+
+    if (storage->setFileName(convertPath(src.path()), dest.fileName()) != 0) {
+        return WorkerResult::fail(ERR_CANNOT_RENAME, src.path());
+    }
+    return WorkerResult::pass();
 }
 
 WorkerResult MTPWorker::fileSystemFreeSpace(const QUrl &url)
@@ -808,4 +839,15 @@ int MTPWorker::waitForCopyOperation(const KMTPStorageInterface *storage)
     return loop.exec();
 }
 
+KIO::WorkerResult MTPWorker::chmod(const QUrl &url, int permissions)
+{
+    return WorkerResult::pass();
+}
+
+KIO::WorkerResult MTPWorker::chown(const QUrl &url, const QString &owner, const QString &group)
+{
+    return WorkerResult::pass();
+}
+
 #include "kio_mtp.moc"
+#include "moc_kio_mtp.cpp"
