@@ -1,24 +1,26 @@
 #ifndef RECENTDOCUMENT_H
 #define RECENTDOCUMENT_H
 
-#include <KIO/ForwardingSlaveBase>
+#include <KIO/ForwardingWorkerBase>
 
 class KDirWatch;
 
-class RecentDocuments : public KIO::ForwardingSlaveBase
+class RecentDocuments : public KIO::ForwardingWorkerBase
 {
 public:
     RecentDocuments(const QByteArray &pool, const QByteArray &app);
     ~RecentDocuments() override;
 
+public:
+    KIO::WorkerResult listDir(const QUrl &url) override;
+    KIO::WorkerResult stat(const QUrl& url) override;
+    KIO::WorkerResult mimetype(const QUrl& url) override;
+
 protected:
     QString desktopFile(KIO::UDSEntry&) const;
+
     bool rewriteUrl(const QUrl &url, QUrl &newUrl) override;
-    void listDir(const QUrl &url) override;
-    void prepareUDSEntry(KIO::UDSEntry &entry, bool listing = false) const override;
-    void stat(const QUrl& url) override;
-    void mimetype(const QUrl& url) override;
-    void del(const QUrl& url, bool isfile) override;
+
 private:
     KDirWatch* m_recentDocWatch;
 };
