@@ -42,6 +42,11 @@ using namespace std::filesystem;
 #include "kio_sftp_debug.h"
 #include "kio_sftp_trace_debug.h"
 
+// For MinGW compatibility
+#ifndef QT_STAT_LNK
+#define QT_STAT_LNK 0120000
+#endif // QT_STAT_LNK
+
 #define KIO_SFTP_SPECIAL_TIMEOUT 30
 
 // How big should each data packet be? Definitely not bigger than 64kb or
