@@ -8,14 +8,8 @@
 
 #include <QObject>
 
-#include <kdnssd_version.h>
-#if KDNSSD_VERSION >= QT_VERSION_CHECK(5, 84, 0)
 #include <KDNSSD/RemoteService>
 #include <KDNSSD/ServiceBrowser>
-#else
-#include <DNSSD/RemoteService>
-#include <DNSSD/ServiceBrowser>
-#endif
 
 #include "discovery.h"
 
