@@ -6,12 +6,10 @@
 
 #include "kio_smb.h"
 #include "smburl.h"
-#include <kconfig.h>
-#include <kconfiggroup.h>
+#include <KConfig>
+#include <KConfigGroup>
 
-#include <QTextCodec>
-
-void SMBSlave::reparseConfiguration()
+void SMBWorker::reparseConfiguration()
 {
     m_context.authenticator()->loadConfiguration();
 }
